@@ -1,5 +1,5 @@
 /*
- * $XFree86: xc/programs/Xserver/render/render.c,v 1.33 2005/10/14 15:17:27 tsi Exp $
+ * $XFree86: xc/programs/Xserver/render/render.c,v 1.37 2008/10/15 20:59:13 tsi Exp $
  *
  * Copyright  2000 SuSE, Inc.
  *
@@ -213,7 +213,7 @@ RenderClientCallback (CallbackListPtr	*list,
 }
 
 void
-RenderExtensionInit (void)
+RenderExtensionInit (INITARGS)
 {
     ExtensionEntry *extEntry;
 
@@ -1069,11 +1069,14 @@ ProcRenderAddGlyphs (ClientPtr client)
     }
 
     nglyphs = stuff->nglyphs;
+    if (nglyphs > (CARD32)(-1L) / sizeof(GlyphNewRec))
+	    return BadAlloc;
+
     if (nglyphs <= NLOCALGLYPH)
 	glyphsBase = glyphsLocal;
     else
     {
-	glyphsBase = (GlyphNewPtr) ALLOCATE_LOCAL (nglyphs * sizeof (GlyphNewRec));
+	glyphsBase = (GlyphNewPtr) Xalloc (nglyphs * sizeof (GlyphNewRec));
 	if (!glyphsBase)
 	    return BadAlloc;
     }
@@ -1131,7 +1134,7 @@ ProcRenderAddGlyphs (ClientPtr client)
     }
 
     if (glyphsBase != glyphsLocal)
-	DEALLOCATE_LOCAL (glyphsBase);
+	Xfree (glyphsBase);
     return client->noClientException;
 bail:
     while (glyphs != glyphsBase)
@@ -1140,7 +1143,7 @@ bail:
 	xfree (glyphs->glyph);
     }
     if (glyphsBase != glyphsLocal)
-	DEALLOCATE_LOCAL (glyphsBase);
+	Xfree (glyphsBase);
     return err;
 }
 
@@ -1454,8 +1457,9 @@ ProcRenderCreateCursor (ClientPtr client)
     pScreen = pSrc->pDrawable->pScreen;
     width = pSrc->pDrawable->width;
     height = pSrc->pDrawable->height;
-    if ( stuff->x > width 
-      || stuff->y > height )
+    if (height && width > ((unsigned int)(-1) / (height * sizeof(CARD32))))
+	return (BadAlloc);
+    if ((stuff->x > width) || (stuff->y > height))
 	return (BadMatch);
     argbbits = xalloc (width * height * sizeof (CARD32));
     if (!argbbits)
@@ -1893,6 +1897,8 @@ SProcRenderSetPictureClipRectangles (ClientPtr client)
     REQUEST(xRenderSetPictureClipRectanglesReq);
     swaps(&stuff->length, n);
     swapl(&stuff->picture, n);
+    swaps(&stuff->xOrigin, n);
+    swaps(&stuff->yOrigin, n);
     SwapRestS(stuff);
     return (*ProcRenderVector[stuff->renderReqType]) (client);
 }
