@@ -25,7 +25,7 @@ other dealings in this Software without prior written authorization
 from The Open Group.
 
 */
-/* $XFree86: xc/programs/xdm/xdmcp.c,v 3.28 2006/01/09 15:01:04 dawes Exp $ */
+/* $XFree86: xc/programs/xdm/xdmcp.c,v 3.29 2006/06/28 02:31:48 dawes Exp $ */
 
 /*
  * xdm - display manager daemon
@@ -576,8 +576,11 @@ NetworkAddressToName(
 	    {
 		if (!strcmp (localhost, hostname))
 		{
-		    if (!getString (name, 10))
-			return 0;
+		    if (!getString (name, 10)) {
+			if (ai)
+			    freeaddrinfo(ai);
+			return NULL;
+		    }
 		    sprintf (name, ":%d", displayNumber);
 		}
 		else
@@ -603,15 +606,21 @@ NetworkAddressToName(
 			}
 		    }
 
-		    if (!getString (name, strlen (hostname) + 10))
-			return 0;
+		    if (!getString (name, strlen (hostname) + 10)) {
+			if (ai)
+			    freeaddrinfo(ai);
+			return NULL;
+		    }
 		    sprintf (name, "%s:%d", hostname, displayNumber);
 		}
 	    }
 	    else
 	    {
-		if (!getString (name, INET6_ADDRSTRLEN + 10))
-		    return 0;
+		if (!getString (name, INET6_ADDRSTRLEN + 10)) {
+		    if (ai)
+			freeaddrinfo(ai);
+		    return NULL;
+		}
 		if (multiHomed) {
 		    if (connectionType == FamilyInternet) {
 			data = (CARD8 *) 
@@ -624,7 +633,9 @@ NetworkAddressToName(
 		}
 		if (inet_ntop(type, data, name, INET6_ADDRSTRLEN) == NULL) {
 		    free(name);
-		    return 0;
+		    if (ai)
+			freeaddrinfo(ai);
+		    return NULL;
 		} 
 		sprintf(name + strlen(name), ":%d", displayNumber);
 	    }
@@ -1424,20 +1435,28 @@ NetworkAddressToHostname (
 		struct addrinfo	*ai = NULL, *nai;
 		if (getaddrinfo(hostent->h_name, NULL, NULL, &ai) == 0) {
 		    for (nai = ai; nai != NULL; nai = nai->ai_next) {
-			if ((af_type == nai->ai_family) &&
-			  (connectionAddress->length == nai->ai_addrlen) &&
-			  (memcmp(connectionAddress->data,nai->ai_addr,
-			    nai->ai_addrlen) == 0) ) {
+			if ((af_type == nai->ai_family) && (
+			  ((nai->ai_family == AF_INET) &&
+			    (connectionAddress->length == sizeof(struct in_addr)) &&
+			    (memcmp(connectionAddress->data,
+				    &((struct sockaddr_in *)nai->ai_addr)->sin_addr,
+				    connectionAddress->length) == 0)) ||
+			  ((nai->ai_family == AF_INET6) &&
+			    (connectionAddress->length == sizeof(struct in6_addr)) &&
+			    (memcmp(connectionAddress->data,
+				    &((struct sockaddr_in6 *)nai->ai_addr)->sin6_addr,
+				    connectionAddress->length) == 0))))
 			    break;
-			}
 		    }
-		    if (ai == NULL) {
+		    if (nai == NULL) {
 			inet_ntop(af_type, connectionAddress->data, 
 			  dotted, sizeof(dotted));
 
-			LogError("Possible DNS spoof attempt %s->%s.", dotted,
+			LogError("Possible DNS spoof attempt %s->%s.\n", dotted,
 			  hostent->h_name);
 			hostent = NULL;
+		    } else {
+		      local_name = hostent->h_name;
 		    }
 		    freeaddrinfo(ai);
 		} else {
@@ -1448,7 +1467,7 @@ NetworkAddressToHostname (
 		if ((hostent = gethostbyname(s))) {
 			if (memcmp((char*)connectionAddress->data, hostent->h_addr,
 			    hostent->h_length) != 0) {
-				LogError("Possible DNS spoof attempt.");
+				LogError("Possible DNS spoof attempt.\n");
 				hostent = NULL; /* so it enters next if() */
 			} else {
 				local_name = hostent->h_name;
